@@ -8,7 +8,7 @@ import RPi.GPIO as GPIO
 from time import sleep, perf_counter
 from statistics import mean, median, stdev
 from .utils import convert_to_list
-import logging
+from logging import getLogger, Logger, StreamHandler
 
 class HX711:
     """
@@ -44,58 +44,83 @@ class HX711:
                  log_level: str = 'WARN',
                  ):
         
-        logging.basicConfig(level=log_level, format='HX711 :: %(asctime)s :: %(levelname)s :: %(message)s')
+        
+        self._logger: Logger = getLogger('hx711-multi')
+        self._logger.setLevel(log_level)
+        consoleLogHandler = StreamHandler()
+        consoleLogHandler.setLevel(log_level)
+        self._logger.addHandler(consoleLogHandler)
         self._all_or_nothing = all_or_nothing
-        self._set_dout_pins(dout_pins)
-        self._set_sck_pin(sck_pin)
+        self._dout_pins = dout_pins
+        self._sck_pin = sck_pin
         # init GPIO before channel because a read operation is required for channel initialization
         self._init_gpio()
-        self._set_channel_a_gain(channel_A_gain)
-        self._set_channel_select(channel_select)
+        self._channel_A_gain = channel_A_gain
+        self._channel_select = channel_select
         self._init_load_cells()
         # perform a read which sets channel and gain
         self._read()
         sleep(0.4) # 400ms settling time according to documentation
-
-    def _set_dout_pins(self, dout_pins):
+        
+    @property
+    def _dout_pins(self):
+        return self.__dout_pins
+    
+    @_dout_pins.setter
+    def _dout_pins(self, dout_pins):
         """ set dout_pins as array of ints. If just an int input, turn it into a single array of int """
         self._single_load_cell = (type(dout_pins) is int)
-        self._dout_pins = convert_to_list(dout_pins, _type=int, _default_output=None)
-        if self._dout_pins is None:
+        _dout_pins_temp = convert_to_list(dout_pins, _type=int, _default_output=None)
+        if _dout_pins_temp is None:
             # raise error if pins not set properly
             raise TypeError(f'dout_pins must be type int or array of int.\nReceived dout_pins: {dout_pins}')
+        self.__dout_pins = _dout_pins_temp
         
-    def _set_sck_pin(self, sck_pin):
-        # set sck_pin if int
+    @property
+    def _sck_pin(self):
+        return self.__sck_pin
+    
+    @_sck_pin.setter
+    def _sck_pin(self, sck_pin):
         if type(sck_pin) is not int:
             raise TypeError(f'sck_pin must be type int.\nReceived sck_pin: {sck_pin}')
-        self._sck_pin = sck_pin
-                    
+        self.__sck_pin = sck_pin
+        
+    @property
+    def _channel_A_gain(self):
+        return self.__channel_A_gain
+    
+    @_channel_A_gain.setter
+    def _channel_A_gain(self, channel_A_gain):
+        # check channel_select for type and value. Default is A if None
+        if channel_A_gain not in [128, 64]:
+            # raise error if channel not 128 or 64
+            raise TypeError(f'channel_A_gain must be A or B.\nReceived channel_A_gain: {channel_A_gain}')
+        self.__channel_A_gain = channel_A_gain
+    
+    @property
+    def _channel_select(self):
+        return self.__channel_select
+    
+    @_channel_select.setter
+    def _channel_select(self, channel_select):
+        # check channel_select for type and value. Default is A if None
+        if channel_select not in ['A', 'B']:
+            # raise error if channel not A or B
+            raise TypeError(f'channel_select must be A or B.\nReceived channel_select: {channel_select}')
+        self.__channel_select = channel_select
+
     def _init_gpio(self):
         # init GPIO
         GPIO.setup(self._sck_pin, GPIO.OUT)  # sck_pin is output only
         for dout in self._dout_pins:
             GPIO.setup(dout, GPIO.IN)  # dout_pin is input only
             
-    def _set_channel_a_gain(self, channel_A_gain):
-        # check channel_select for type and value. Default is A if None
-        if channel_A_gain not in [128, 64]:
-            # raise error if channel not 128 or 64
-            raise TypeError(f'channel_A_gain must be A or B.\nReceived channel_A_gain: {channel_A_gain}')
-        self._channel_A_gain = channel_A_gain
-        
-    def _set_channel_select(self, channel_select):
-        # check channel_select for type and value. Default is A if None
-        if channel_select not in ['A', 'B']:
-            # raise error if channel not A or B
-            raise TypeError(f'channel_select must be A or B.\nReceived channel_select: {channel_select}')
-        self._channel_select = channel_select
-                
     def _init_load_cells(self):
         # initialize load cell instances
         self._load_cells = []
         for dout_pin in self._dout_pins:
-            self._load_cells.append(LoadCell(dout_pin))
+            self._load_cells.append(LoadCell(dout_pin=dout_pin, logger=self._logger))
 
     def _prepare_to_read(self):
         """
@@ -118,9 +143,9 @@ class HX711:
                 sleep(0.01)
         ready = all([load_cell._ready for load_cell in self._load_cells])
         if ready:
-            logging.debug(f'checked sensor readiness, completed after {i+1} iterations')
+            self._logger.debug(f'checked sensor readiness, completed after {i+1} iterations')
         else:
-            logging.warn(f'checked sensor readiness, not ready after {i+1} iterations')              
+            self._logger.warn(f'checked sensor readiness, not ready after {i+1} iterations')              
         return ready
     
     def _pulse_sck_high(self):
@@ -138,7 +163,7 @@ class HX711:
         # check if pulse lasted 60ms or longer. If so, HX711 enters power down mode
         if pulse_end - pulse_start >= 0.00006:  # check if the hx 711 did not turn off...
             # if pd_sck pin is HIGH for 60 us and more than the HX 711 enters power down mode.
-            logging.warn(f'sck pulse lasted for longer than 60us\nTime elapsed: {pulse_end - pulse_start}')
+            self._logger.warn(f'sck pulse lasted for longer than 60us\nTime elapsed: {pulse_end - pulse_start}')
             return False
         return True
     
@@ -239,12 +264,12 @@ class HX711:
                 load_cell._calculate_measurement()
             
         all_load_cell_vars = "\n".join([str(vars(load_cell)) for load_cell in self._load_cells])
-        logging.info(f'Finished read operation. Load cell results:\n{all_load_cell_vars}')
+        self._logger.info(f'Finished read operation. Load cell results:\n{all_load_cell_vars}')
 
-        load_cell_measurements = [load_cell.measurement_from_offset for load_cell in self._load_cells]
+        load_cell_measurements = [load_cell.measurement_from_zero for load_cell in self._load_cells]
 
         if not load_cell_measurements or all(x is None for x in load_cell_measurements):
-            logging.critical(f'All load cell measurements failed. '
+            self._logger.warning(f'All load cell measurements failed. '
                 'This is either due to all load cells actually failing, '
                 'or if you have set all_or_nothing=True and 1 or more load calls failed')
 
@@ -312,8 +337,8 @@ class HX711:
         load_cell: LoadCell
         for load_cell in self._load_cells:
             if load_cell._ready:
-                logging.debug(f'zeroing with {len(load_cell._reads_filtered)} datapoints')
-                load_cell.zero_from_mean()
+                self._logger.debug(f'zeroing with {len(load_cell._reads_filtered)} datapoints')
+                load_cell.zero_from_last_measurement()
 
     def set_weight_multiples(self, weight_multiples, load_cell_indices = None, dout_pins = None):
         """
@@ -354,10 +379,12 @@ class LoadCell:
     """
     
     def __init__(self,
-                 dout_pin,
+                 dout_pin: int,
+                 logger: Logger,
                  ):
         self._dout_pin = dout_pin
-        self._offset = 0.
+        self._logger = logger
+        self._zero_offset = 0.
         self._weight_multiple = 1.
         self._ready = False
         self._current_raw_read = 0
@@ -370,17 +397,24 @@ class LoadCell:
         self._devs_from_med = []
         self._read_stdev = 0.
         self._ratios_to_stdev = []
-        self.measurement = None # mean of reads
-        self.measurement_from_offset = None # measurement minus offset
-        self.weight = None # measurement_from_offset divided by weight_multiple
-        
-    def zero_from_mean(self):
+        self.measurement = None # mean value of raw reads after filtering
+        self.measurement_from_zero = None # measurement minus offset
+        self.weight = None # measurement_from_zero divided by weight_multiple
+    
+    def zero_from_last_measurement(self):
         """ sets offset based on current value for measurement """
         if self.measurement:
-            self._offset = self.measurement
+            self.zero(self.measurement)
         else:
             raise ValueError(f'Trying to zero LoadCell (dout={self._dout_pin}) with a bad mean value. '
                               'Value of measurement: {self.measurement}')
+    
+    def zero(self, offset: float = None):
+        """ sets offset based on current value for measurement """
+        if offset is not None:
+            self._zero_offset = offset
+        else:
+            raise ValueError(f'No offset provided to zero() function')
         
     def set_weight_multiple(self, weight_multiple: float):
         """ simply sets multiple. example: scale indicates value of 5000 for 1 gram on scale, weight_multiple = 5000 """
@@ -396,7 +430,7 @@ class LoadCell:
         self._read_stdev = 0.
         self._ratios_to_stdev = []
         self.measurement = None
-        self.measurement_from_offset = None
+        self.measurement_from_zero = None
         self.weight = None
         self._init_raw_read()
     
@@ -424,13 +458,13 @@ class LoadCell:
         self._current_signed_value = self.convert_to_signed_value(self._current_raw_read)
         self.reads.append(self._current_signed_value)
         # log 2's complement value and signed value
-        logging.debug(f'Binary value: {bin(self._current_raw_read)} -> Signed: {self._current_signed_value}')
+        self._logger.debug(f'Binary value: {bin(self._current_raw_read)} -> Signed: {self._current_signed_value}')
             
     def convert_to_signed_value(self, raw_value):
         # convert to signed value after verifying value is valid
         # raise error if value is exactly the min or max value, or a value of all 1's
         if raw_value in [0x800000, 0x7FFFFF, 0xFFFFFF]:
-            logging.debug('Invalid raw value detected: {}'.format(hex(raw_value)))
+            self._logger.debug('Invalid raw value detected: {}'.format(hex(raw_value)))
             return None  # return None because the data is invalid
         # calculate int from 2's complement
         # check if the sign bit is 1, indicating a negative number
@@ -471,7 +505,7 @@ class LoadCell:
             # if standard deviation is too large, the scale isn't actually ready
             # sometimes with a bad scale connection, the bit will come back ready out of chance and the binary values are garbage data
             self._ready = False
-            logging.warn(f'load cell (dout {self._dout_pin}) not ready, stdev from median was over {self._max_stdev}: {self._read_stdev}')
+            self._logger.warn(f'load cell (dout {self._dout_pin}) not ready, stdev from median was over {self._max_stdev}: {self._read_stdev}')
         elif self._read_stdev:
             self._ratios_to_stdev = [(dev / self._read_stdev) for dev in self._devs_from_med]
         else:
@@ -489,7 +523,7 @@ class LoadCell:
             # no values after filter, so return False to indicate no read value
             return False
         self.measurement = mean(self._reads_filtered)
-        self.measurement_from_offset = self.measurement - self._offset
-        self.weight = self.measurement_from_offset / self._weight_multiple
+        self.measurement_from_zero = self.measurement - self._zero_offset
+        self.weight = self.measurement_from_zero / self._weight_multiple
         
         return True
\ No newline at end of file
